
// ColourFloatWin.h

#ifndef ColourFloatWin_h
#define ColourFloatWin_h

//#include "tflowndw.h"
//#include "czctrls.h"
//#include <vector>
//#include <algorithm>
//#include "gdwindows.h"
//#include "clayer.h"
#include "ObjectScrollView.h"
#include "GraphicsContext.h"
//#include "DocColorModel.h"
#include "NativeScrollView.h"
#include "GDCntrls.h"
#include "NativeSlider.h"
#include "NativeText.h"
#include "NativeStepper.h"
//#include "InksControl.h"
#include "Inks.h"
#include "gdwindows.h"


class TDocument;


class
	InkSlider:
public
	gd::ChildGraphicsContext
{
private:
	long m_Min;
	long m_Max;
	long m_Value;
	long m_DrawWidth;
public:
	InkSlider( GraphicsContext* parent, long hOffset, long vOffset, long min, long max,
		long value )
		: ChildGraphicsContext( parent, hOffset, vOffset ),
		m_Min( min ), m_Max( max ), m_Value( value ), m_DrawWidth( 50 )
	{}
	InkSlider( const InkSlider& other )
		: ChildGraphicsContext( other )
	{
		m_Min = other.m_Min;
		m_Max = other.m_Max;
		m_Value = other.m_Value;
		m_DrawWidth = other.m_DrawWidth;
	}

	void SetValues( long min, long max, long value )
	{
		assert( value >= 0 );
		assert( min >= 0 );

		m_Min = min;
		m_Max = max;
		m_Value = value;
	}
	void Draw();
	bool DoContent( gd::ipoint where, CoordType coordType )
	{
		assert( m_Value >= 0 );
		assert( m_Min >= 0 );

		gd::ipoint p = where;
		if ( coordType == ParentCoords )
			ParentToSelf( p );

		if ( p.y < 0 || p.y > 11 || p.x < 0 )
			return false;

		m_Value = m_Min + ( ( p.x * ( m_Max - m_Min ) ) / m_DrawWidth );
		m_Value = std::max< long >( std::min< long >( m_Value, m_Max ), m_Min );
		Draw();

		while ( StillDown() )
		{
			gd::ipoint mousePos = GetMouse();
			long newValue = m_Min + ( ( mousePos.x * ( m_Max - m_Min ) ) / m_DrawWidth );
			newValue = std::max< long >( std::min< long >( newValue, m_Max ), m_Min );
			if ( newValue != m_Value ) {
				m_Value = newValue;
				Draw();
			}
		}

		return true;
	}
	long Value()
	{
		assert( m_Value >= 0 );
		assert( m_Min >= 0 );

		return m_Value;
	}
	const InkSlider& operator =( const InkSlider& other )
	{
		m_Min = other.m_Min;
		m_Max = other.m_Max;
		m_Value = other.m_Value;
		m_DrawWidth = other.m_DrawWidth;
		
		return *this;
	}
};

#pragma warning( disable : 4355 )

class
	InkLineObj:
public
	LineObject
{
private:
	InkSpecPtr		ink;
	Shade			shade;
	//InkShade		m_InkShade;
	InkSlider		m_Slider;

public:
	InkLineObj( GraphicsContext* parent, long hOffset, long vOffset, long width,
		const InkSpecPtr& inkParam, Shade shadeParam )
		: LineObject( parent, hOffset, vOffset, width, 12 ), ink( inkParam ), shade( shadeParam ),
		m_Slider( this, 100, 0, 0, 255, shade.Value100() )
	{
	}

	void Draw()
	{
		PaintRect( gd::irect( 0, 0, 12, 12 ), ink->GetFullRGB() );
		MoveTo( gd::Point( 15, 10 ) );

		DrawString( ink->Name(), rgb_black, 9 );

		//char shadeStr[ 50 ];
		//itoa( m_InkShade.second, shadeStr, 10 );
		//MoveTo( Point( 155, 10 ) );
		//DrawString( shadeStr, rgb_black, 9 );

		//m_Slider.SetValues( 0, 100, m_InkShade.second );
		m_Slider.Draw();
	}
	bool DoContent( gd::ipoint where, CoordType coordType )
	{
		gd::ipoint p = where;

		if ( coordType == ParentCoords )
			ParentToSelf( p );

		if ( m_Slider.DoContent( p, coordType ) ) {
			shade.SetValue100( m_Slider.Value() );
			return true;
		}
		return false;
	}
	long Value()
	{
		return m_Slider.Value();
	}
	void OnMouseClicked( bool value ) {}
};


class ColorizeApp;


class InksControl;
class InksControlHolder;


class
	ColourFloatWin :
public
	gd::GDWindow_Floating,
	ScrollerWatcher
{
private:
	InksControlHolder*  inksControlHolder;
	gd::NativeScrollBar scrollBar;

	void ArrangeControls();
public:
	ColourFloatWin( ColorizeApp* app, const std::wstring& winName, const gd::irect& sizeRect );
	~ColourFloatWin();

	void			SetInkColour( const MultiInkColor* inkColor, const MultiInkColor* inkColor2, long documentId, long layerId );
	MultiInkColor	InkColour();
	MultiInkColor	InkColour2();
	void			InkColourFromControls();

	void Draw( const gd::irect& drawArea );
	//void Init( const gd::irect& sizeRect, const unsigned short* winName, HWND wndClient );
	void DoContent( gd::ipoint where );
	void DocumentChanged();
	void SetColorFromCurrentDocument();
	void TextChanged();
	void Init();
	void InksControlHasCHanged();
	WindowAttributes WinAttributes();
	void DrawBlend();
	void NativeControlPressed( long controlID );
	void ScrollerPosChanged( long newPostion );
	void AdjustScrollRange();
};


#endif

